@@ -1,18 +1,19 @@
 package Devel::Caller::Util;
 
-our $DATE = '2015-07-26'; # DATE
-our $VERSION = '0.03'; # VERSION
+our $DATE = '2019-04-12'; # DATE
+our $VERSION = '0.040'; # VERSION
 
-use 5.010001;
 use warnings;
 use strict;
 
 use Exporter qw(import);
-our @EXPORT_OK = qw(callers);
+our @EXPORT_OK = qw(caller callers);
+
+my $_is_caller;
 
 sub callers {
-    my $start = $_[0] // 0;
-    my $with_args = $_[1];
+    my ($start, $with_args, $packages_to_ignore, $subroutines_to_ignore) = @_;
+    $start = 0 unless defined $start;
 
     my @res;
     my $i = $start+1;
@@ -21,7 +22,8 @@ sub callers {
         my @caller;
         if ($with_args) {
             {
-                package DB;
+                package # hide from PAUSE
+                    DB;
                 @caller = caller($i);
                 $caller[11] = [@DB::args] if @caller;
             }
@@ -29,14 +31,36 @@ sub callers {
             @caller = caller($i);
         }
         last unless @caller;
-
-        push @res, \@caller;
         $i++;
+
+        if ($packages_to_ignore) {
+            if (ref $packages_to_ignore eq 'ARRAY') {
+                next if grep { $_ eq $caller[0] } @$packages_to_ignore;
+            } else {
+                next if $caller[0] =~ $packages_to_ignore;
+            }
+        }
+
+        if ($subroutines_to_ignore) {
+            if (ref $subroutines_to_ignore eq 'ARRAY') {
+                next if grep { $_ eq $caller[3] } @$subroutines_to_ignore;
+            } else {
+                next if $caller[3] =~ $subroutines_to_ignore;
+            }
+        }
+
+        do { $_is_caller = 0; return @caller } if $_is_caller;
+        push @res, \@caller;
     }
 
     @res;
 }
 
+sub caller {
+    $_is_caller = 1; # "local" doesn't work here
+    goto &callers;
+}
+
 1;
 # ABSTRACT: caller()-related utility routines
 
@@ -52,17 +76,23 @@ Devel::Caller::Util - caller()-related utility routines
 
 =head1 VERSION
 
-This document describes version 0.03 of Devel::Caller::Util (from Perl distribution Devel-Caller-Util), released on 2015-07-26.
+This document describes version 0.040 of Devel::Caller::Util (from Perl distribution Devel-Caller-Util), released on 2019-04-12.
 
 =head1 SYNOPSIS
 
- use Devel::Util::Caller qw(callers);
+ use Devel::Util::Caller qw(caller callers);
+
+ my @info = caller(3);
 
  my @callers = callers();
 
 =head1 FUNCTIONS
 
-=head2 callers([ $start=0 [, $with_args] ]) => LIST
+=head2 caller([ $offset [, $with_args [, $packages_to_ignore [, $subroutines_to_ignore ] ] ] ]) => LIST
+
+Just like the built-in C<caller()>
+
+=head2 callers([ $start=0 [, $with_args [, $packages_to_ignore [, $subroutines_to_ignore ] ] ] ]) => LIST
 
 A convenience function to return the whole callers stack, produced by calling
 C<caller()> repeatedly from frame C<$start+1> until C<caller()> returns empty.
@@ -78,7 +108,10 @@ Result will be like:
 If C<$with_args> is true, will also return subroutine arguments in the 11th
 element of each frame, produced by retrieving C<@DB::args>.
 
-=head1 SEE ALSO
+C<$packages_to_ignore> can be set to a regex (will be matched against
+C<$packageI>) or an arrayref of package names. Similarly,
+C<$subroutines_to_ignore> can be set to a regex or an arrayref of subroutine
+names. Note that subroutine names are B<fully qualified names>.
 
 =head1 HOMEPAGE
 
@@ -96,13 +129,15 @@ When submitting a bug or request, please include a test-file or a
 patch to an existing test-file that illustrates the bug or desired
 feature.
 
+=head1 SEE ALSO
+
 =head1 AUTHOR
 
 perlancar <perlancar@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2015 by perlancar@cpan.org.
+This software is copyright (c) 2019, 2015 by perlancar@cpan.org.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
